@@ -8,7 +8,7 @@
 
 #define MAX_LEN 200
 
-char *in_word_set (char *, int);
+char *in_word_set (const char *, int);
 
 int 
 main (int argc, char *argv[])
