@@ -22,38 +22,17 @@ and this notice must be preserved on all copies.
 */
 
 #ifndef _File_h 
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 #define _File_h 1
 
 #include <builtin.h>
 #include <stdio.h>
 #include <stddef.h>
 
-enum io_mode                    // known unix file IO modes
-{
-  io_readonly   = 0,            
-  io_writeonly  = 1,
-  io_readwrite  = 2, 
-  io_appendonly = 3,
-  io_append     = 4,            // append, plus allow reads
-};
-
-enum access_mode                // ways to open a file
-{
-  a_createonly  = 0,            // create, fail if file exists
-  a_create      = 1,            // create if doesn't exist, else truncate
-  a_useonly     = 2,            // use (no truncate)  fail if doesn't exist
-  a_use         = 3,            // use (no truncate), create if doesn't exist
-};
-
-enum state_value                // File states
-{ 
-  _good         = 0,            // all is well
-  _eof          = 1,            // at eof
-  _fail         = 2,            // logical or physical IO error
-  _bad          = 4             // unopened/corrupted
-};
-
+#include <Fmodes.h>
 
 class Filebuf;
 
@@ -167,13 +146,13 @@ public:
 
 // error handlers
 
-extern void  verbose_File_error_handler(char*);
-extern void  quiet_File_error_handler(char*);
-extern void  fatal_File_error_handler(char*);
+extern void  verbose_File_error_handler(const char*);
+extern void  quiet_File_error_handler(const char*);
+extern void  fatal_File_error_handler(const char*);
 extern one_arg_error_handler_t File_error_handler;
 extern one_arg_error_handler_t set_File_error_handler(one_arg_error_handler_t);
 
-//#ifdef __OPTIMIZE__
+#if defined(__OPTIMIZE__) || defined(USE_LIBGXX_INLINES)
 
 
 
@@ -230,18 +209,6 @@ inline File& File::raw()
   return this->File::setbuf(_IONBF); 
 }
 
-inline void File::check_state() // ensure fp & state agree about eof
-{
-  if (fp != 0)
-  {
-    if (feof(fp))
-      set(_eof);
-    else
-      unset(_eof);
-    if (ferror(fp))
-      set(_bad);
-  }
-}
 
 inline File& File::failif(int cond)
 { 
@@ -291,12 +258,20 @@ inline File& File::flush()
 
 inline File& File::flush(char ch)
 { 
+#ifdef VMS
+  return failif(!is_open() || c$$flsbuf(ch, fp) == EOF);
+#else
   return failif(!is_open() || _flsbuf(ch, fp) == EOF);
+#endif
 }
 
 inline int File::fill()
 { 
+#ifdef VMS
+  failif(!is_open() || (stat = c$$filbuf(fp)) == EOF);
+#else
   failif(!is_open() || (stat = _filbuf(fp)) == EOF);
+#endif
   return stat;
 }
 
@@ -342,6 +317,6 @@ inline int File::good()
 }
 
 
-//#endif
+#endif
 
 #endif
