@@ -33,7 +33,10 @@
    they can distinguish the relative order of options and other arguments.  */
 
 #ifndef GetOpt_h
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 #define GetOpt_h 1
 
 #include <std.h>
