@@ -1,5 +1,8 @@
 #ifndef Incremental_h
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 #define Incremental_h
 #define DECLARE_INIT_FUNCTION(USER_INIT_FUNCTION) \
 static void USER_INIT_FUNCTION (); extern void (*_initfn)(); \
