@@ -28,7 +28,10 @@ and this notice must be preserved on all copies.
 */
 
 #ifndef _PlotFile_h
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 #define _PlotFile_h
 
 #include <File.h>
@@ -45,33 +48,55 @@ protected:
   PlotFile& cmd(char c);
   PlotFile& operator << (const int x);
   PlotFile& operator << (const char *s);
-
+  
 public:
-                             File::close;     File::raw;       
-            File::remove;    File::filedesc;  File::is_open;
-            File::iocount;   File::error;     File::name;
-            File::setname;   File::rdstate;
-            File::eof;       File::fail;      File::bad;
-            File::good;      File::clear;     File::failif;
-                             File::writable;  File::readable;
-
-            PlotFile();
-            PlotFile(const char* filename, io_mode m, access_mode a);
-            PlotFile(const char* filename, const char* m);
-            PlotFile(int filedesc, io_mode m = io_writeonly);
-            PlotFile(FILE* fileptr);
-
-           ~PlotFile();
-
-            operator void*();
-
-  PlotFile& open(const char* filename, const io_mode m, const access_mode a);
+  
+  PlotFile();
+  PlotFile(const char* filename, io_mode m, access_mode a);
+  PlotFile(const char* filename, const char* m);
+  PlotFile(int filedesc, const io_mode m = io_writeonly);
+  PlotFile(FILE* fileptr);
+  
+  ~PlotFile();
+  
+  operator void*();
+  
+  PlotFile& close() { File::close(); return *this; }
+  PlotFile& remove() { File::remove(); return *this; }
+  
+  int           filedesc() { return File::filedesc(); }
+  const char*   name() { return File::name(); }
+  void          setname(const char* newname) { File::setname(newname); }
+  int           iocount() { return File::iocount(); }
+  
+  int           rdstate() { return File::rdstate(); }
+  int           eof() { return File::eof(); }
+  int           fail() { return File::fail(); }
+  int           bad() { return File::bad(); }
+  int           good() { return File::good(); }
+  
+  // other status queries
+  
+  int           readable() { return File::readable(); }
+  int           writable() { return File::writable(); }
+  int           is_open() { return File::is_open(); }
+  
+  void          error() {  File::error(); }
+  void          clear(state_value f = _good) {  File::clear(f); }
+  void          set(state_value f) { File::set(f); }
+  void          unset(state_value f) { File::unset(f); }
+  PlotFile&     failif(int cond) {  File::failif(cond); return *this; }
+  void          check_state() { File::check_state(); }
+  
+  PlotFile&     raw() { File::raw(); return *this; }
+  
+  PlotFile& open(const char* filename, io_mode m, access_mode a);
   PlotFile& open(const char* filename, const char* m);
-  PlotFile& open(const int  filedesc, const io_mode m);
+  PlotFile& open(int  filedesc,  io_mode m);
   PlotFile& open(FILE* fileptr);
   PlotFile& setbuf(const int buffer_kind); // vals: _IONBF, _IOFBF, _IOLBF
   PlotFile& setbuf(const int size, char* buf);
-
+  
   PlotFile& arc(const int xi, const int yi,
                 const int x0, const int y0,
                 const int x1, const int y1);
@@ -94,3 +119,36 @@ public:
 
 
 #endif
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
