@@ -21,7 +21,10 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  
 */
 #ifndef _Poisson_h
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 #define _Poisson_h 
 
 #include <Random.h>
@@ -38,7 +41,7 @@ public:
     virtual double operator()();
 };
 
-//#ifdef __OPTIMIZE__
+#if defined(__OPTIMIZE__) || defined(USE_LIBGXX_INLINES)
 
 inline Poisson::Poisson(double mean, RNG *gen)
 : (gen) {
@@ -52,5 +55,6 @@ inline double Poisson::mean(double x) {
   return t;
 }
 
-//#endif
+
+#endif
 #endif
