@@ -21,9 +21,15 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  
 */
 #ifndef _Random_h
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 #define _Random_h 1
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 
 #include <math.h>
 #include "RNG.h"
@@ -39,6 +45,8 @@ public:
     void generator(RNG *p);
 };
 
+#if defined(__OPTIMIZE__) || defined(USE_LIBGXX_INLINES)
+
 inline Random::Random(RNG *gen)
 {
     pGenerator = gen;
@@ -55,3 +63,5 @@ inline void Random::generator(RNG *p)
 }
 
 #endif
+
+#endif
