@@ -22,15 +22,26 @@ and this notice must be preserved on all copies.
 */
 
 #ifndef _stddef_h
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 
 #define _stddef_h
 
 // TRUE, FALSE, bool, are now in <bool.h>, for ANSI compatibility
 
+#ifndef _SIZE_T
+#ifndef _SIZE_T_
+typedef unsigned int size_t;
+#endif
+#endif
 
-typedef unsigned long   size_t;
-typedef long int	    ptrdiff_t;
+#ifndef _PTRDIFF_T
+#ifndef _PTRDIFF_T_
+typedef long int	 ptrdiff_t;
+#endif
+#endif
 
 #ifdef DGUX
 typedef	int	wchar_t;
@@ -42,5 +53,8 @@ typedef	unsigned short	wchar_t;
 #define NULL 0
 #endif
 
+/* Offset of member MEMBER in a struct of type TYPE.  */
+
+#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
 
 #endif
