@@ -4,7 +4,11 @@
 
 extern "C"
 {
+#ifdef VMS
+#include "GNU_CC_INCLUDE:[sys]stat.h"
+#else
 #include "//usr/include/sys/stat.h"
+#endif
 
 #ifndef stat_h
 #define stat_h 1
