@@ -1,12 +1,16 @@
 #ifndef wait_h
 
-#include <resource.h>
+#include <sys/resource.h>
 
 #define wait WaitStatus
 
 extern "C" {
 
+#ifdef VMS
+#include "GNU_CC_INCLUDE:[sys]wait.h"
+#else
 #include "//usr/include/sys/wait.h"
+#endif
 
 #undef wait
 
