@@ -23,7 +23,10 @@ and this notice must be preserved on all copies.
 
 
 #ifndef _values_h
+#ifdef __GNUG__
 #pragma once
+#pragma interface
+#endif
 #define _values_h 1
 
 #define BITSPERBYTE 8
@@ -45,9 +48,25 @@ and this notice must be preserved on all copies.
 #define MAXINT      (~MININT)
 #define MAXLONG     (~MINLONG)
 
-#if defined(sun) || defined(hp300) || defined(hpux)
+#define HIBITS	MINSHORT
+#define HIBITL	MINLONG
+
+#if defined(sun) || defined(hp300) || defined(hpux) || defined(masscomp)
+#ifdef masscomp
+#define MAXDOUBLE							\
+({									\
+  double maxdouble_val;							\
+									\
+  __asm ("fmove%.d #0x7fefffffffffffff,%0"	/* Max double */	\
+	 : "=f" (maxdouble_val)						\
+	 : /* no inputs */);						\
+  maxdouble_val;							\
+})
+#define MAXFLOAT ((float) 3.40e+38)
+#else
 #define MAXDOUBLE   1.79769313486231470e+308
 #define MAXFLOAT    ((float)3.40282346638528860e+38)
+#endif
 #define MINDOUBLE   4.94065645841246544e-324
 #define MINFLOAT    ((float)1.40129846432481707e-45)
 #define _IEEE       1
