@@ -28,6 +28,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "read-line.h"
 #include "hash-table.h"
 #include "key-list.h"
+#include "trace.h"
 
 /* Efficiently returns the least power of two greater than or equal to X! */
 #define POW(X) ((!X)?1:(X-=1,X|=X>>1,X|=X>>2,X|=X>>4,X|=X>>8,X|=X>>16,(++X)))
@@ -39,6 +40,7 @@ static int field_width = 0;
 
 Key_List::~Key_List (void) 
 { 
+  T (Trace t ("Key_List::~Key_List");)
   if (option[DEBUG])
     {
       fprintf (stderr, "\nDumping key list information:\ntotal non-static linked keywords = %d"
@@ -66,6 +68,7 @@ Key_List::~Key_List (void)
 char *
 Key_List::get_special_input (char delimiter)
 { 
+  T (Trace t ("Key_List::get_special_input");)
   int size  = 80;
   char *buf = new char[size];
   int c, i;
@@ -113,12 +116,13 @@ Key_List::get_special_input (char delimiter)
 char *
 Key_List::save_include_src (void)
 {
+  T (Trace t ("Key_List::save_include_src");)
   int c;
   
   if ((c = getchar ()) != '%')
     ungetc (c, stdin);
   else if ((c = getchar ()) != '{')
-    report_error ("internal error, %c != '{' on line %d in file %s%a", c, __LINE__, __FILE__);
+    report_error ("internal error, %c != '{' on line %d in file %s%a", c, __LINE__, __FILE__, 1);
   else 
     return get_special_input ('}');
   return "";
@@ -131,6 +135,7 @@ Key_List::save_include_src (void)
 char *
 Key_List::get_array_type (void)
 {
+  T (Trace t ("Key_List::get_array_type");)
   return get_special_input ('%');
 }  
   
@@ -141,6 +146,7 @@ Key_List::get_array_type (void)
 static inline int
 Key_List::strcspn (const char *s, const char *reject)
 {
+  T (Trace t ("Key_List::strcspn");)
   const char *scan;
   const char *rej_scan;
   int   count = 0;
@@ -164,6 +170,7 @@ Key_List::strcspn (const char *s, const char *reject)
 void 
 Key_List::set_output_types (void)
 {
+  T (Trace t ("Key_List::set_output_types");)
   if (option[TYPE] && !(array_type = get_array_type ()))
     return;                     /* Something's wrong, bug we'll catch it later on.... */
   else if (option[TYPE])        /* Yow, we've got a user-defined type... */
@@ -193,6 +200,7 @@ Key_List::set_output_types (void)
 void 
 Key_List::read_keys (void)
 {
+  T (Trace t ("Key_List::read_keys");)
   char *ptr;
   
   include_src = save_include_src ();
@@ -201,7 +209,7 @@ Key_List::read_keys (void)
   /* Oops, problem with the input file. */  
   if (! (ptr = Read_Line::get_line ()))
     report_error ("No words in input file, did you forget to prepend %s"
-                  " or use -t accidentally?\n%a", "%%");
+                  " or use -t accidentally?\n%a", "%%", 1);
   
   /* Read in all the keywords from the input file. */
   else 
@@ -269,8 +277,8 @@ Key_List::read_keys (void)
       /* Exit program if links exists and option[DUP] not set, since we can't continue */
       if (total_duplicates)
         report_error (option[DUP]
-                      ? "Some input keys have identical hash values, examine output carefully...\n"
-                      : "Some input keys have identical hash values,\ntry different key positions or use option -D.\n%a");
+                      ? "%d input keys have identical hash values, examine output carefully...\n"
+                      : "%d input keys have identical hash values,\ntry different key positions or use option -D.\n%a", total_duplicates, 1);
       if (option[ALLCHARS])
         option.set_keysig_size (max_key_len);
     }
@@ -285,6 +293,7 @@ Key_List::read_keys (void)
 List_Node *
 Key_List::merge (List_Node *list1, List_Node *list2)
 {
+  T (Trace t ("Key_List::merge");)
   if (!list1)
     return list2;
   else if (!list2)
@@ -308,6 +317,7 @@ Key_List::merge (List_Node *list1, List_Node *list2)
 List_Node *
 Key_List::merge_sort (List_Node *head)
 { 
+  T (Trace t ("Key_List::merge_sort");)
   if (!head || !head->next)
     return head;
   else
@@ -334,10 +344,10 @@ Key_List::merge_sort (List_Node *head)
 static inline int 
 Key_List::get_occurrence (List_Node *ptr)
 {
-  int   value = 0;
-  char *temp;
+  T (Trace t ("Key_List::get_occurrence");)
+  int value = 0;
 
-  for (temp = ptr->char_set; *temp; temp++)
+  for (char *temp = ptr->char_set; *temp; temp++)
     value += occurrences[*temp];
   
   return value;
@@ -349,11 +359,9 @@ Key_List::get_occurrence (List_Node *ptr)
 static inline void 
 Key_List::set_determined (List_Node *ptr)
 {
-  char *temp;
-  
-  for (temp = ptr->char_set; *temp; temp++)
+  T (Trace t ("Key_List::set_determined");)
+  for (char *temp = ptr->char_set; *temp; temp++)
     determined[*temp] = 1;
-  
 }
 
 /* Returns TRUE if PTR's key set is already completely determined. */
@@ -361,10 +369,10 @@ Key_List::set_determined (List_Node *ptr)
 static inline int  
 Key_List::already_determined (List_Node *ptr)
 {
-  int   is_determined = 1;
-  char *temp;
+  T (Trace t ("Key_List::already_determined");)
+  int is_determined = 1;
 
-  for (temp = ptr->char_set; is_determined && *temp; temp++)
+  for (char *temp = ptr->char_set; is_determined && *temp; temp++)
     is_determined = determined[*temp];
   
   return is_determined;
@@ -379,13 +387,11 @@ Key_List::already_determined (List_Node *ptr)
 void 
 Key_List::reorder (void)
 {
-  List_Node *ptr;
-
-  for (ptr = head; ptr; ptr = ptr->next)
+  T (Trace t ("Key_List::reorder");)
+  for (List_Node *ptr = head; ptr; ptr = ptr->next)
     ptr->occurrence = get_occurrence (ptr);
   
-  hash_sort       = 0;
-  occurrence_sort = 1;
+  occurrence_sort = !(hash_sort = 0); /* Pretty gross, eh?! */
   
   for (ptr = head = merge_sort (head); ptr->next; ptr = ptr->next)
     {
@@ -421,15 +427,14 @@ Key_List::reorder (void)
 void
 Key_List::output_min_max ()
 {
-  List_Node *temp;
-
-  for (temp = head; temp->next; temp = temp->next)
+  T (Trace t ("Key_List::output_min_max");)
+  for (List_Node *temp = head; temp->next; temp = temp->next)
     ;
 
   min_hash_value = head->hash_value;
   max_hash_value = temp->hash_value;
 
-  if (option[C])
+  if (!option[ENUM])
     printf ("\n#define TOTAL_KEYWORDS %d\n#define MIN_WORD_LENGTH %d"
             "\n#define MAX_WORD_LENGTH %d\n#define MIN_HASH_VALUE %d"
             "\n#define MAX_HASH_VALUE %d\n", total_keys, min_key_len,
@@ -451,6 +456,7 @@ Key_List::output_min_max ()
 void 
 Key_List::output_switch (void)
 {
+  T (Trace t ("Key_List::output_switch");)
   char      *comp_buffer;
   List_Node *curr                     = head;
   int        pointer_and_type_enabled = option[POINTER] && option[TYPE];
@@ -491,7 +497,7 @@ Key_List::output_switch (void)
           while (temp->next && temp->hash_value == temp->next->hash_value)
             temp = temp->next;
 
-      if (temp)
+      if (temp && total_switches != 1)
         printf ("          if (key <= %d)\n            {\n", temp->hash_value);
       else
         printf ("            {\n");
@@ -506,7 +512,13 @@ Key_List::output_switch (void)
                   option[CONST] ? "const " : "",
                   pointer_and_type_enabled ? struct_tag : "char",
                   option[LENTABLE] && !option[DUP] ? "int key_len;" : "");
-          printf ("              switch (key - %d)\n                {\n", lowest_case_value);
+          if (total_switches == 1)
+            {
+              printf ("              switch (key)\n                {\n");
+              lowest_case_value = 0;
+            }
+          else
+            printf ("              switch (key - %d)\n                {\n", lowest_case_value);
 
           for (temp = curr; temp && ++i <= number_of_cases; temp = temp->next)
             {
@@ -602,6 +614,7 @@ Key_List::output_switch (void)
 void 
 Key_List::output_keylength_table (void)
 {
+  T (Trace t ("Key_List::output_keylength_table");)
   const int  max_column = 15;
   int        index      = 0;
   int        column     = 0;
@@ -634,21 +647,25 @@ Key_List::output_keylength_table (void)
 void 
 Key_List::output_keyword_table (void)
 {
+  T (Trace t ("Key_List::output_keyword_table");)
   char      *l_brace = *head->rest ? "{" : "";
   char      *r_brace = *head->rest ? "}," : "";
-  char      *indent       = option[GLOBAL] ? "" : "  ";
-  int        index = 0;
+  char      *indent  = option[GLOBAL] ? "" : "  ";
+  int        index   = 0;
   List_Node *temp;
 
-  printf ("\n%sstatic %s%swordlist[] =\n%s%s{\n",
+  printf ("%sstatic %s%swordlist[] =\n%s%s{\n",
           indent, option[CONST] ? "const " : "", struct_tag, indent, indent);
 
   /* Skip over leading blank entries if there are no duplicates. */
 
-  if (total_duplicates == 0)
-    while (index < head->hash_value)
-      index++;
-  
+  printf ("      ");
+  for (int column = 1; index < head->hash_value; index++, column++)
+    printf ("%s\"\",%s %s", l_brace, r_brace, column % 9 ? "" : "\n      ");      
+  if (column % 10)
+    printf ("\n");
+
+          
   /* Generate an array of reserved words at appropriate locations. */
   
   for (temp = head ; temp; temp = temp->next, index++)
@@ -668,22 +685,30 @@ Key_List::output_keyword_table (void)
             printf ("\n");
           else 
             {
-              printf ("%s\"%s\", %s%s\n", l_brace, temp->key, temp->rest, r_brace);
+              printf ("%s\"%s\", %s%s", l_brace, temp->key, temp->rest, r_brace);
+              if (option[DEBUG])
+                printf (" /* hash value = %d, index = %d */", temp->hash_value, temp->index);
+              putchar ('\n');
               continue;
             }
         }
 
-      printf ("      %s\"%s\", %s%s\n", l_brace, temp->key, temp->rest, r_brace);
+      printf ("      %s\"%s\", %s%s", l_brace, temp->key, temp->rest, r_brace);
+      if (option[DEBUG])
+        printf (" /* hash value = %d, index = %d */", temp->hash_value, temp->index);
+      putchar ('\n');
 
       /* Deal with links specially. */
       if (temp->link)
         for (List_Node *links = temp->link; links; links = links->link)
           {
             links->index = ++index;
-            printf ("      %s\"%s\", %s%s\n", l_brace, links->key, links->rest, r_brace);
+            printf ("      %s\"%s\", %s%s", l_brace, links->key, links->rest, r_brace);
+            if (option[DEBUG])
+              printf (" /* hash value = %d, index = %d */", links->hash_value, links->index);
+            putchar ('\n');
           }
     }
-
   printf ("%s%s};\n\n", indent, indent);
 }
 
@@ -693,6 +718,7 @@ Key_List::output_keyword_table (void)
 void 
 Key_List::output_hash_function (void)
 {
+  T (Trace t ("Key_List::output_hash_function");)
   const int max_column  = 10;
   int       count       = max_hash_value;
 
@@ -798,140 +824,134 @@ Key_List::output_hash_function (void)
 void
 Key_List::output_lookup_array (void)
 {
-  const int DEFAULT_VALUE = -1;
+  T (Trace t ("Key_List::output_lookup_array");)
+  if (total_duplicates > 0)
+    {
+      const int DEFAULT_VALUE = -1;
 
-  struct dup
-    {
-      int hash_value;           /* Hash value for this particular duplicate set. */
-      int index;                /* Index into the main keyword storage array. */
-      int count;                /* Number of consecutive duplicates at this index. */
-    };
-  dup duplicates[total_duplicates], *dup_ptr = duplicates;
-  int lookup_array[max_hash_value + 1], *lookup_ptr = lookup_array + max_hash_value + 1;
-  List_Node *temp;
-  
-  while (lookup_ptr > lookup_array)
-    *--lookup_ptr = DEFAULT_VALUE;
-  
-  for (temp = head; temp; temp = temp->next)
-    {
-      lookup_array[temp->hash_value] = temp->index;
-      if (option[DEBUG])
-        fprintf (stderr, "keyword = %s, index = %d\n", temp->key, temp->index);
-      if (temp->link)
+      struct 
         {
-          List_Node *ptr;
-          lookup_array[temp->hash_value] = MIN_SIGNED_SHORT;
-          *dup_ptr = (struct dup){ temp->hash_value, temp->index, 1 };
+          int hash_value;       /* Hash value for this particular duplicate set. */
+          int index;            /* Index into the main keyword storage array. */
+          int count;            /* Number of consecutive duplicates at this index. */
+        } duplicates[total_duplicates], *dup_ptr = duplicates;
+      int lookup_array[max_hash_value + 1], *lookup_ptr = lookup_array + max_hash_value + 1;
+
+      while (lookup_ptr > lookup_array)
+        *--lookup_ptr = DEFAULT_VALUE;
+  
+      for (List_Node *temp = head; temp; temp = temp->next)
+        {
+          int hash_value = temp->hash_value;
+          lookup_array[hash_value] = temp->index;
+          if (option[DEBUG])
+            fprintf (stderr, "keyword = %s, index = %d\n", temp->key, temp->index);
+          if (!temp->link &&
+              (!temp->next || hash_value != temp->next->hash_value))
+            continue;
+          *dup_ptr = (typeof (*dup_ptr)) { hash_value, temp->index, 1 };
           
-          for (ptr = temp->link; ptr; ptr = ptr->link)
+          for (List_Node *ptr = temp->link; ptr; ptr = ptr->link)
             {
               dup_ptr->count++;
               if (option[DEBUG])
-                fprintf (stderr, "keyword = %s, index = %d\n", ptr->key, ptr->index);            
+                fprintf (stderr, "static linked keyword = %s, index = %d\n", ptr->key, ptr->index);            
             }
-        }
-      if (temp->next && temp->hash_value == temp->next->hash_value)
-        {
-          if (lookup_array[temp->hash_value] != MIN_SIGNED_SHORT)
-            {
-              lookup_array[temp->hash_value] = MIN_SIGNED_SHORT;
-              *dup_ptr = (struct dup){ temp->hash_value, temp->index, 1 };
-            }
-          
-          for ( ; temp->next && temp->hash_value == temp->next->hash_value;
-               temp = temp->next)
+
+          while (temp->next && hash_value == temp->next->hash_value)
             {
+              temp = temp->next;
               dup_ptr->count++;
               if (option[DEBUG])
-                fprintf (stderr, "keyword = %s, index = %d\n", temp->key, temp->index);
+                fprintf (stderr, "dynamic linked keyword = %s, index = %d\n", temp->key, temp->index);
+
+              for (List_Node *ptr = temp->link; ptr; ptr = ptr->link)
+                {
+                  dup_ptr->count++;
+                  if (option[DEBUG])
+                    fprintf (stderr, "static linked keyword = %s, index = %d\n", ptr->key, ptr->index);            
+                }
             }
           dup_ptr++;
         }
-      else if (temp->link)
-        dup_ptr++;
-    }
   
-  while (--dup_ptr >= duplicates)
-    {
-      int i;
-      
-      if (option[DEBUG])
-        fprintf (stderr, "dup_ptr[%d]: hash_value = %d, index = %d, count = %d\n",
-                dup_ptr - duplicates, dup_ptr->hash_value, dup_ptr->index, dup_ptr->count);
-
-      /* Start searching for available space towards the right part of the lookup array. */
-      for (i = dup_ptr->hash_value; i < max_hash_value; i++)
-        if (lookup_array[i] == DEFAULT_VALUE && lookup_array[i + 1] == DEFAULT_VALUE)
-          {
-            lookup_array[i] = -dup_ptr->index;
-            lookup_array[i + 1] = -dup_ptr->count;
-            lookup_array[dup_ptr->hash_value] = total_keys + (i - dup_ptr->hash_value);
-            break;
-          }
-
-      /* If we didn't find it to the right look to the left instead... */
-      if (i == max_hash_value)
+      while (--dup_ptr >= duplicates)
         {
+          if (option[DEBUG])
+            fprintf (stderr, "dup_ptr[%d]: hash_value = %d, index = %d, count = %d\n",
+                     dup_ptr - duplicates, dup_ptr->hash_value, dup_ptr->index, dup_ptr->count);
 
-          for (i = dup_ptr->hash_value; i > 0; i--)
-            if (lookup_array[i] == DEFAULT_VALUE && lookup_array[i - 1] == DEFAULT_VALUE)
+          /* Start searching for available space towards the right part of the lookup array. */
+          for (int i = dup_ptr->hash_value; i < max_hash_value; i++)
+            if (lookup_array[i] == DEFAULT_VALUE && lookup_array[i + 1] == DEFAULT_VALUE)
               {
-                lookup_array[i - 1] = -dup_ptr->index;
-                lookup_array[i] = -dup_ptr->count;
-                lookup_array[dup_ptr->hash_value] = -(total_keys + (dup_ptr->hash_value - i + 1));
+                lookup_array[i] = -dup_ptr->index;
+                lookup_array[i + 1] = -dup_ptr->count;
+                lookup_array[dup_ptr->hash_value] = max_hash_value + (i - dup_ptr->hash_value);
                 break;
               }
 
-          /* We are in big trouble if this happens! */
-          assert (i != 0);
+          /* If we didn't find it to the right look to the left instead... */
+          if (i == max_hash_value)
+            {
+
+              for (i = dup_ptr->hash_value; i > 0; i--)
+                if (lookup_array[i] == DEFAULT_VALUE && lookup_array[i - 1] == DEFAULT_VALUE)
+                  {
+                    lookup_array[i - 1] = -dup_ptr->index;
+                    lookup_array[i] = -dup_ptr->count;
+                    lookup_array[dup_ptr->hash_value] = -(max_hash_value + (dup_ptr->hash_value - i + 1));
+                    break;
+                  }
+
+              /* We are in *big* trouble if this happens! */
+              assert (i != 0);
+            }
         }
+
+      int max = MIN_INT;
+
+      for (lookup_ptr = lookup_array + max_hash_value + 1; lookup_ptr > lookup_array; max >?= abs (*--lookup_ptr))
+        ;
+
+      char *indent = option[GLOBAL] ? "" : "  ";
+      printf ("%sstatic %s%s lookup[] =\n%s%s{\n      ", indent, option[CONST] ? "const " : "",
+              max <= MAX_SIGNED_CHAR ? "char" : (max <= MAX_SIGNED_SHORT ? "short" : "int"),
+              indent, indent);
+
+      int count = max;
+
+      /* Calculate maximum number of digits required for MAX_HASH_VALUE. */
+
+      for (field_width = 2; (count /= 10) > 0; field_width++)
+        ;
+
+      const int max_column = 15;
+      int column = 0;
+
+      for (lookup_ptr = lookup_array; 
+           lookup_ptr < lookup_array + max_hash_value + 1;
+           lookup_ptr++)
+        printf ("%*d,%s", field_width, *lookup_ptr, ++column % (max_column - 1) ? "" : "\n      ");
+
+      printf ("\n%s%s};\n\n", indent, indent);
     }
-
-  int max = NEG_MAX_INT;
-
-  for (lookup_ptr = lookup_array + max_hash_value + 1; lookup_ptr > lookup_array; max >?= abs (*--lookup_ptr))
-    ;
-
-  printf ("  static %s%s lookup[] =\n    {\n      ", option[CONST] ? "const " : "",
-          max <= MAX_UNSIGNED_CHAR ? "char" : (max <= MAX_UNSIGNED_SHORT ? "short" : "int"));
-
-  int count = max;
-
-  /* Calculate maximum number of digits required for MAX_HASH_VALUE. */
-
-  for (field_width = 2; (count /= 10) > 0; field_width++)
-    ;
-
-  /* Skip over the leading default entries. */
-  for (lookup_ptr = lookup_array; lookup_ptr < lookup_array + min_hash_value; lookup_ptr++)
-    assert (*lookup_ptr == DEFAULT_VALUE);
-
-  assert (*lookup_ptr != DEFAULT_VALUE);
-
-  const int max_column = 15;
-        int column     = 0;
-
-  for ( ; lookup_ptr < lookup_array + max_hash_value + 1; lookup_ptr++)
-    printf ("%*d,%s", field_width, *lookup_ptr, ++column % (max_column - 1) ? "" : "\n      ");
-
-  printf ("\n    };\n\n");
 }
-
 /* Generates C code to perform the keyword lookup. */
 
 void 
 Key_List::output_lookup_function (void)
 { 
+  T (Trace t ("Key_List::output_lookup_function");)
   printf ("  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)\n    {\n"
-          "      register int key = %s (str, len) - MIN_HASH_VALUE;\n\n"
-          "      if (key <= (MAX_HASH_VALUE - MIN_HASH_VALUE) && key >= 0)\n        {\n",
+          "      register int key = %s (str, len);\n\n"
+          "      if (key <= MAX_HASH_VALUE && key >= 0)\n        {\n",
           option.get_hash_name ());
   
-  if (option[DUP]) 
+  if (option[DUP] && total_duplicates > 0)
     {
       printf ("          register int index = lookup[key];\n\n"
-              "          if (index >= 0 && index < TOTAL_KEYWORDS)\n"
+              "          if (index >= 0 && index < MAX_HASH_VALUE)\n"
               "            {\n"
               "              register %schar *s = wordlist[index]", option[CONST] ? "const " : "");
       if (array_type != default_array_type)
@@ -940,10 +960,10 @@ Key_List::output_lookup_function (void)
       printf (";\n\n              if (%s*s == *str && !%s)\n                return %s;\n            }\n",
               option[LENTABLE] ? "len == lengthtable[key]\n              && " : "",
               option[COMP] ? "strncmp (str + 1, s + 1, len - 1)" : "strcmp (str + 1, s + 1)",
-              option[TYPE] && option[POINTER] ? "&wordlist[key]" : "s");
-      printf ("          else if (index < 0 && index >= -TOTAL_KEYWORDS)\n"
+              option[TYPE] && option[POINTER] ? "&wordlist[index]" : "s");
+      printf ("          else if (index < 0 && index >= -MAX_HASH_VALUE)\n"
               "            return 0;\n          else\n            {\n"
-              "              register int offset = key + index + (index > 0 ? -TOTAL_KEYWORDS : TOTAL_KEYWORDS);\n"
+              "              register int offset = key + index + (index > 0 ? -MAX_HASH_VALUE : MAX_HASH_VALUE);\n"
               "              register %s%s*base = &wordlist[-lookup[offset]];\n"
               "              register %s%s*ptr = base + -lookup[offset + 1];\n\n"
               "              while (--ptr >= base)\n                ",
@@ -980,7 +1000,7 @@ Key_List::output_lookup_function (void)
 void 
 Key_List::output (void)
 {
-  int global_table = option[GLOBAL];       
+  T (Trace t ("Key_List::output");)
   printf ("%s\n", include_src);
   
   if (option[TYPE] && !option[NOTYPE]) /* Output type declaration now, reference it later on.... */
@@ -989,16 +1009,15 @@ Key_List::output (void)
   output_min_max ();
 
   if (option[CPLUSPLUS])
-    printf ("class %s\n{\nprivate:\n  const int MIN_WORD_LENGTH, MAX_WORD_LENGTH, MIN_HASH_VALUE, MAX_HASH_VALUE;\n"
-            "  unsigned int hash (const char *str, int len);\npublic:\n  %s (void):\n"
-            "    MIN_WORD_LENGTH (%d), MAX_WORD_LENGTH (%d), MIN_HASH_VALUE (%d), MAX_HASH_VALUE (%d) {}\n"
-            "  %soperator () (const char *str, int len);\n};\n\n",
-            option.get_class_name (), option.get_class_name (),
-            min_key_len, max_key_len, min_hash_value, max_hash_value, return_type);
+    printf ("class %s\n{\nprivate:\n"
+            "  static unsigned int hash (const char *str, int len);\npublic:\n"
+            "  static %s%s%s (const char *str, int len);\n};\n\n",
+            option.get_class_name (), option[CONST] ? "const " : "", 
+            return_type, option.get_function_name ());
 
   output_hash_function ();
   
-  if (global_table)
+  if (option[GLOBAL])
     if (option[SWITCH])
       {
         if (option[LENTABLE] && option[DUP])
@@ -1011,6 +1030,7 @@ Key_List::output (void)
         if (option[LENTABLE])
           output_keylength_table ();
         output_keyword_table ();
+        output_lookup_array ();
       }
   
   if (option[GNU])          /* Use the inline keyword to remove function overhead. */
@@ -1023,12 +1043,22 @@ Key_List::output (void)
   printf (option[ANSI] 
           ? "%s (register const char *str, register int len)\n{\n"
           : "%s (str, len)\n     register char *str;\n     register unsigned int len;\n{\n",
-          option[CPLUSPLUS] ? "operator ()" : option.get_function_name ());
-  
+          option.get_function_name ());
+
+  if (option[ENUM])
+    printf ("  enum\n    {\n"
+            "      TOTAL_KEYWORDS = %d,\n"
+            "      MIN_WORD_LENGTH = %d,\n"
+            "      MAX_WORD_LENGTH = %d,\n"
+            "      MIN_HASH_VALUE = %d,\n"
+            "      MAX_HASH_VALUE = %d,\n"
+            "    };\n\n",
+            total_keys, min_key_len, max_key_len, min_hash_value, max_hash_value);
+
   /* Use the switch in place of lookup table. */
   if (option[SWITCH])
     {               
-      if (!global_table)
+      if (!option[GLOBAL])
         {
           if (option[LENTABLE] && option[DUP])
             output_keylength_table ();
@@ -1040,13 +1070,13 @@ Key_List::output (void)
   /* Use the lookup table, in place of switch. */
   else                
     {           
-      if (!global_table)
+      if (!option[GLOBAL])
         {
           if (option[LENTABLE])
             output_keylength_table ();
           output_keyword_table ();
         }
-      if (total_duplicates)
+      if (!option[GLOBAL])
         output_lookup_array ();
       output_lookup_function ();
     }
@@ -1063,6 +1093,7 @@ Key_List::output (void)
 void 
 Key_List::sort (void) 
 { 
+  T (Trace t ("Key_List::sort");)
   hash_sort       = 1;
   occurrence_sort = 0;
   
@@ -1074,6 +1105,7 @@ Key_List::sort (void)
 void 
 Key_List::dump () 
 {      
+  T (Trace t ("Key_List::dump");)
   int field_width = option.get_max_keysig_size ();
   
   fprintf (stderr, "\nList contents are:\n(hash value, key length, index, %*s, keyword):\n",
@@ -1089,8 +1121,9 @@ Key_List::dump ()
 
 Key_List::Key_List (void) 
 {   
+  T (Trace t ("Key_List::Key_List");)
   total_keys       = 1;
-  max_key_len      = NEG_MAX_INT;
+  max_key_len      = MIN_INT;
   min_key_len      = MAX_INT;
   return_type      = default_return_type;
   array_type       = struct_tag  = default_array_type;
@@ -1104,6 +1137,7 @@ Key_List::Key_List (void)
 int 
 Key_List::keyword_list_length (void) 
 { 
+  T (Trace t ("Key_List::keyword_list_length");)
   return list_len;
 }
 
@@ -1112,6 +1146,7 @@ Key_List::keyword_list_length (void)
 int 
 Key_List::max_key_length (void) 
 { 
+  T (Trace t ("Key_List::max_key_length");)
   return max_key_len;
 }
 
