@@ -25,21 +25,22 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <time.h>
+#include <stdio.h>
 #include "std-err.h"
 #include "options.h"
 #include "gen-perf.h"
+#include "trace.h"
 
 int
 main (int argc, char *argv[])
 {
+  T (Trace t ("main");)
   struct tm *tm;
   time_t     clock; 
-  int        status;
 
   time (&clock);
   tm = localtime (&clock);
-  fprintf (stderr, "/* starting time is %d:%02d:%02d */\n",
-           tm->tm_hour, tm->tm_min, tm->tm_sec);
+  printf ("/* starting time is %d:%02d:%02d */\n", tm->tm_hour, tm->tm_min, tm->tm_sec);
 
 #ifdef UNLIMIT_STACK
   /* Get rid of any avoidable limit on stack size.  */
@@ -61,11 +62,11 @@ main (int argc, char *argv[])
 
   /* Generates and prints the Gen_Perf hash table.
      Don't use exit here, it skips the destructors. */
-  status = generate_table ();     
+  int status = generate_table ();     
 
   time (&clock);
   tm = localtime (&clock);
-  fprintf (stderr, "/* ending time is %d:%02d:%02d */\n",
-           tm->tm_hour, tm->tm_min, tm->tm_sec);
+  printf ("/* ending time is %d:%02d:%02d */\n", tm->tm_hour, tm->tm_min, tm->tm_sec);
+
   return status;
 }
