@@ -36,7 +36,8 @@ extern char *version_string;
 static inline void 
 Options::usage (void) 
 { 
-  report_error ("Usage: %n [-acCdDef[num]gGhH<hashname>i<init>jk<keys>K<keyname>lL<language>nN<name>oprs<size>S<switches>tTv].\n"
+  T (Trace t ("Options::usage");)
+  report_error ("Usage: %n [-acCdDef[num]gGhH<hashname>i<init>jk<keys>K<keyname>lL<language>nN<function name>oprs<size>S<switches>tTvZ<class name>].\n"
                 "(type %n -h for help)\n");
 }
 
@@ -45,6 +46,7 @@ Options::usage (void)
 void 
 Options::print_options (void)
 { 
+  T (Trace t ("Options::print_options");)
   int i;
 
   printf ("/* Command-line: ");
@@ -63,6 +65,7 @@ Options::print_options (void)
 static inline int 
 Options::key_sort (char *base, int len) 
 {
+  T (Trace t ("Options::key_sort");)
   int i, j;
 
   for (i = 0, j = len - 1; i < j; i++) 
@@ -83,6 +86,7 @@ Options::key_sort (char *base, int len)
 
 Options::Options (void) 
 { 
+  T (Trace t ("Options::Options");)
   key_positions[0]    = WORD_START;
   key_positions[1]    = WORD_END;
   key_positions[2]    = EOS;
@@ -102,6 +106,7 @@ Options::Options (void)
 
 Options::~Options (void) 
 { 
+  T (Trace t ("Options::~Options");)
   if (option_word & DEBUG)
     {
       char *ptr;
@@ -112,7 +117,7 @@ Options::~Options (void)
                "\nPOINTER is.....: %s\nNOLENGTH is....: %s\nLENTABLE is....: %s"
                "\nDUP is.........: %s\nFAST is........: %s\nCOMP is.....: %s"
                "\nNOTYPE is......: %s\nGLOBAL is......: %s\nCONST is....: %s"
-               "\nCPLUSPLUS is...: %s\nC is...........: %s"
+               "\nCPLUSPLUS is...: %s\nC is...........: %s\nENUM is.....: %s"
                "\niterations = %d\nlookup function name = %s\nhash function name = %s"
                "\nkey name = %s\njump value = %d\nmax associcated value = %d"
                "\ninitial associated value = %d\ndelimiters = %s\nnumber of switch statements = %d\n",
@@ -135,6 +140,7 @@ Options::~Options (void)
                option_word & CONST ? "enabled" : "disabled",
                option_word & CPLUSPLUS ? "enabled" : "disabled",
                option_word & C ? "enabled" : "disabled",
+               option_word & ENUM ? "enabled" : "disabled",
                iterations, function_name, hash_name, key_name, jump, size - 1,
                initial_asso_value, delimiters, total_switches);
       if (option_word & ALLCHARS) 
@@ -157,16 +163,17 @@ Options::~Options (void)
 /* Parses the command line Options and sets appropriate flags in option_word. */
 
 void 
-Options::operator () (int argc, char *argv[])
+Options::operator() (int argc, char *argv[])
 { 
-  GetOpt getopt (argc, argv, "adcCDe:f:gGhH:i:j:k:K:lL:nN:oprs:S:tTv");
+  T (Trace t ("Options::operator()");)
+  GetOpt getopt (argc, argv, "adcCDe:Ef:gGhH:i:j:k:K:lL:nN:oprs:S:tTvZ:");
   int    option_char;
 
   set_program_name (argv[0]);
   argument_count  = argc;
   argument_vector = argv;
 
-  while ((option_char = getopt ()) != EOF)
+  while ((option_char = getopt ()) != -1)
     {
       switch (option_char)
         {
@@ -202,6 +209,11 @@ Options::operator () (int argc, char *argv[])
             option.delimiters = getopt.optarg;
             break;
           }
+        case 'E':
+          {
+            option_word |= ENUM;
+            break;
+          }
         case 'f':               /* Generate the hash table ``fast.'' */
           {
             option_word |= FAST;
@@ -233,6 +245,8 @@ Options::operator () (int argc, char *argv[])
                           "\tfor certain highly redundant keyword sets.  It enables the -S option.\n"
                           "-e\tAllow user to provide a string containing delimiters used to separate\n"
                           "\tkeywords from their attributes.  Default is \",\\n\"\n"
+                          "-E\tDefine constant values using an enum local to the lookup function\n"
+                          "\trather than with defines\n"
                           "-f\tGenerate the gen-perf.hash function ``fast.''  This decreases GPERF's\n"
                           "\trunning time at the cost of minimizing generated table-size.\n"
                           "\tThe numeric argument represents the number of times to iterate when\n"
@@ -292,14 +306,15 @@ Options::operator () (int argc, char *argv[])
                           "\tgroup of fields per line.\n"
                           "-T\tPrevents the transfer of the type declaration to the output file.\n"
                           "\tUse this option if the type is already defined elsewhere.\n"
-                          "-v\tPrints out the current version number\n%e%a\n",
-                          DEFAULT_JUMP_VALUE, (MAX_KEY_POS - 1), usage);
+                          "-v\tPrints out the current version number\n"
+                          "-Z\tAllow user to specify name of generated C++ class.  Default\n"
+                          "\tname is `Perfect_Hash.'\n%e%a", DEFAULT_JUMP_VALUE, (MAX_KEY_POS - 1), usage, 1);
+          }
+        case 'H':               /* Sets the name for the hash function */
+          {
+            hash_name = getopt.optarg;
+            break;
           }
-                                case 'H':               /* Sets the name for the hash function */
-                                        {
-                                                hash_name = getopt.optarg;
-                                                break;
-                                        }
         case 'i':               /* Sets the initial value for the associated values array. */
           { 
             if ((initial_asso_value = atoi (getopt.optarg)) < 0) 
@@ -311,7 +326,7 @@ Options::operator () (int argc, char *argv[])
         case 'j':               /* Sets the jump value, must be odd for later algorithms. */
           { 
             if ((jump = atoi (getopt.optarg)) < 0)
-              report_error ("Jump value %d must be a positive number.\n%e%a", jump, usage);
+              report_error ("Jump value %d must be a positive number.\n%e%a", jump, usage, 1);
             else if (jump && even (jump)) 
               report_error ("Jump value %d should be odd, adding 1 and continuing...\n", jump++);
             break;
@@ -331,16 +346,16 @@ Options::operator () (int argc, char *argv[])
                 for (key_pos = key_positions; (value = expand ()) != EOS; key_pos++) 
                   if (value == BAD_VALUE)
                     report_error ("Illegal key value or range, use 1,2,3-%d,'$' or '*'.\n%e%a",
-                                   MAX_KEY_POS - 1, usage);
+                                   MAX_KEY_POS - 1, usage, 1);
                   else 
                     *key_pos = value;;
 
                 *key_pos = EOS;
 
                 if (! (total_keysig_size = (key_pos - key_positions))) 
-                  report_error ("No keys selected.\n%e%a", usage);
+                  report_error ("No keys selected.\n%e%a", usage, 1);
                 else if (! key_sort (key_positions, total_keysig_size))
-                  report_error ("Duplicate keys selected\n%e%a", usage);
+                  report_error ("Duplicate keys selected\n%e%a", usage, 1);
 
                 if (total_keysig_size != 2 
                     || (key_positions[0] != 1 || key_positions[1] != WORD_END))
@@ -379,7 +394,7 @@ Options::operator () (int argc, char *argv[])
           }
         case 'N':               /* Make generated lookup function name be optarg */
           { 
-            class_name = function_name = getopt.optarg;
+            function_name = getopt.optarg;
             break;
           }
         case 'o':               /* Order input by frequency of key set occurrence. */
@@ -409,7 +424,7 @@ Options::operator () (int argc, char *argv[])
           { 
             option_word |= SWITCH;
             if ((option.total_switches = atoi (getopt.optarg)) <= 0)
-              report_error ("number of switches %s must be a positive number\n%e%a", getopt.optarg, usage);
+              report_error ("number of switches %s must be a positive number\n%e%a", getopt.optarg, usage, 1);
             break;
           }
         case 't':               /* Enable the TYPE mode, allowing arbitrary user structures. */
@@ -423,17 +438,23 @@ Options::operator () (int argc, char *argv[])
                                                 break;
                                         }
         case 'v':               /* Print out the version and quit. */
-          report_error ("%n: version %s\n%e%a\n", version_string, usage);
+          report_error ("%n: version %s\n%e\n%a", version_string, usage, 1);
+        case 'Z':               /* Set the class name. */
+          {
+            class_name = getopt.optarg;
+            break;
+          }
         default: 
-          report_error ("%e%a", usage);
+          report_error ("%e%a", usage, 1);
         }
+      
     }
 
   if (argv[getopt.optind] && ! freopen (argv[getopt.optind], "r", stdin))
-    report_error ("Unable to read key word file %s.\n%e%a", argv[getopt.optind], usage);
+    report_error ("Cannot open keyword file %p\n%e%a", argv[getopt.optind], usage, 1);
   
   if (++getopt.optind < argc) 
-    report_error ("Extra trailing arguments to %n.\n%e%a", usage);
+    report_error ("Extra trailing arguments to %n.\n%e%a", usage, 1);
 }
 
 #ifndef __OPTIMIZE__
@@ -442,6 +463,7 @@ Options::operator () (int argc, char *argv[])
 int  
 Options::operator[] (Option_Type option) 
 { 
+  T (Trace t ("Options::operator[]");)
   return option_word & option;
 }
 
@@ -449,6 +471,7 @@ Options::operator[] (Option_Type option)
 void
 Options::operator= (enum Option_Type opt) 
 {
+  T (Trace t ("Options::operator=");)
         option_word |= opt;
 }
 
@@ -456,6 +479,7 @@ Options::operator= (enum Option_Type opt)
 void
 Options::operator!= (enum Option_Type opt) 
 {
+  T (Trace t ("Options::operator!=");)
         option_word &= ~opt;
 }
 
@@ -463,6 +487,7 @@ Options::operator!= (enum Option_Type opt)
 void 
 Options::reset (void) 
 { 
+  T (Trace t ("Options::reset");)
   key_pos = 0;
 }
 
@@ -470,6 +495,7 @@ Options::reset (void)
 int 
 Options::get (void) 
 {
+  T (Trace t ("Options::get");)
   return key_positions[key_pos++];
 }
 
@@ -477,6 +503,7 @@ Options::get (void)
 void 
 Options::set_asso_max (int r) 
 {
+  T (Trace t ("Options::set_asso_max");)
   size = r;
 }
 
@@ -484,6 +511,7 @@ Options::set_asso_max (int r)
 int 
 Options::get_asso_max (void) 
 {
+  T (Trace t ("Options::get_asso_max");)
   return size;
 }
 
@@ -491,6 +519,7 @@ Options::get_asso_max (void)
 int 
 Options::get_max_keysig_size (void) 
 {
+  T (Trace t ("Options::get_max_keysig_size");)
   return total_keysig_size;
 }
 
@@ -498,6 +527,7 @@ Options::get_max_keysig_size (void)
 void
 Options::set_keysig_size (int size) 
 { 
+  T (Trace t ("Options::set_keysig_size");)
   total_keysig_size = size;
 }
 
@@ -505,6 +535,7 @@ Options::set_keysig_size (int size)
 int 
 Options::get_jump (void) 
 {
+  T (Trace t ("Options::get_jump");)
   return jump;
 }
 
@@ -512,6 +543,7 @@ Options::get_jump (void)
 const char *
 Options::get_function_name (void) 
 {
+  T (Trace t ("Options::get_function_name");)
   return function_name;
 }
 
@@ -519,6 +551,7 @@ Options::get_function_name (void)
 const char *
 Options::get_key_name (void) 
 {
+  T (Trace t ("Options::get_key_name");)
   return key_name;
 }
 
@@ -526,6 +559,7 @@ Options::get_key_name (void)
 const char *
 Options::get_hash_name (void) 
 {
+  T (Trace t ("Options::get_hash_name");)
         return hash_name;
 }
 
@@ -533,6 +567,7 @@ Options::get_hash_name (void)
 const char  *
 Options::get_class_name (void)
 {
+  T (Trace t ("Options::get_class_name");)
   return class_name;
 }
 
@@ -540,6 +575,7 @@ Options::get_class_name (void)
 int 
 Options::initial_value (void) 
 {
+  T (Trace t ("Options::initial_value");)
   return initial_asso_value;
 }
 
@@ -547,6 +583,7 @@ Options::initial_value (void)
 int 
 Options::get_iterations (void) 
 { 
+  T (Trace t ("Options::get_iterations");)
   return iterations;
 }
 
@@ -554,6 +591,7 @@ Options::get_iterations (void)
 const char *
 Options::get_delimiter () 
 {
+  T (Trace t ("Options::get_delimiter");)
   return delimiters;
 }
 
@@ -561,6 +599,7 @@ Options::get_delimiter ()
 int
 Options::get_total_switches () 
 {
+  T (Trace t ("Options::get_total_switches");)
   return total_switches;
 }
 
