@@ -31,6 +31,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #pragma once
 #include "std-err.h"
+#include "trace.h"
 
 /* Enumerate the potential debugging Options. */
 
@@ -56,6 +57,7 @@ enum Option_Type
   CONST        = 0400000,       /* Make the generated tables readonly (const). */
   CPLUSPLUS    = 01000000,      /* Generate C++ code. */
   C            = 02000000,      /* Generate C code. */
+  ENUM	       = 04000000,	/* Use enum for constants. */
 };
 
 /* Define some useful constants (these don't really belong here, but I'm
@@ -133,108 +135,124 @@ extern Options option;
 inline int  
 Options::operator[] (Option_Type option) /* True if option enable, else false. */
 { 
+  T (Trace t ("Options::operator[]");)
   return option_word & option;
 }
 
 inline void
 Options::operator = (enum Option_Type opt) /* Enables option OPT. */
 {
-        option_word |= opt;
+  option_word |= opt;
 }
 
 inline void
 Options::operator != (enum Option_Type opt) /* Disables option OPT. */
 {
-        option_word &= ~opt;
+  option_word &= ~opt;
 }
 
 inline void 
 Options::reset (void) /* Initializes the key Iterator. */
 { 
+  T (Trace t ("Options::reset");)
   key_pos = 0;
 }
 
 inline int 
 Options::get (void) /* Returns current key_position and advanced index. */
 { 
+  T (Trace t ("Options::get");)
   return key_positions[key_pos++];
 }
 
 inline void 
 Options::set_asso_max (int r) /* Sets the size of the table size. */
 { 
+  T (Trace t ("Options::set_asso_max");)
   size = r;
 }
 
 inline int 
 Options::get_asso_max (void) /* Returns the size of the table size. */
 { 
+  T (Trace t ("Options::get_asso_max");)
   return size;
 }
 
 inline int 
 Options::get_max_keysig_size (void) /* Returns total distinct key positions. */
 { 
+  T (Trace t ("Options::get_max_keysig_size");)
   return total_keysig_size;
 }
 
 inline void
 Options::set_keysig_size (int size) /* Sets total distinct key positions. */
 { 
+  T (Trace t ("Options::set_keysig_size");)
   total_keysig_size = size;
 }
 
 inline int 
 Options::get_jump (void) /* Returns the jump value. */
 { 
+  T (Trace t ("Options::get_jump");)
   return jump;
 }
 
 inline const char *
-Options::get_function_name (void) /* Returns the jump value. */
+Options::get_function_name (void) /* Returns the generated function name. */
 { 
+  T (Trace t ("Options::get_function_name");)
   return function_name;
 }
 
 inline const char *
 Options::get_key_name (void) /* Returns the keyword key name. */
 {
+  T (Trace t ("Options::get_key_name");)
   return key_name;
 }
 
 inline const char *
 Options::get_hash_name (void) /* Returns the hash function name. */
 {
+  T (Trace t ("Options::get_hash_name");)
   return hash_name;
 }
 
 inline const char *
 Options::get_class_name (void) /* Returns the generated class name. */
 {
+  T (Trace t ("Options::get_class_name");)
   return class_name;
 }
 
 inline int 
 Options::initial_value (void) /* Returns the initial associated character value. */
 { 
+  T (Trace t ("Options::initial_value");)
   return initial_asso_value;
 }
 
 inline int 
 Options::get_iterations (void) /* Returns the iterations value. */
 { 
+  T (Trace t ("Options::get_iterations");)
   return iterations;
 }
 
 inline const char *
 Options::get_delimiter () /* Returns the string used to delimit keywords from other attributes. */
 {
+  T (Trace t ("Options::get_delimiter");)
   return delimiters;
 }
 
 inline int
 Options::get_total_switches () /* Gets the total number of switch statements to generate. */
 {
+  T (Trace t ("Options::get_total_switches");)
   return total_switches;
 }
 #endif
