@@ -24,6 +24,9 @@ and this notice must be preserved on all copies.
   BitSet class implementation
  */
 
+#ifdef __GNUG__
+#pragma implementation
+#endif
 #include <BitSet.h>
 #include <std.h>
 #include <values.h>
@@ -244,19 +247,6 @@ int operator < (const BitSet& x, const BitSet& y)
     return y.rep->virt;
 }
 
-BitSet shorttoBitSet(unsigned short w) return r
-{
-  r.rep = BitSetalloc(0, &w, 1, 0, 2);  trim(r.rep);
-}
-
-BitSet longtoBitSet(unsigned long w) return r;
-{
-  unsigned short u[2];
-  u[0] = w & ((unsigned short)(~(0)));
-  u[1] = w >> BITSETBITS;
-  r.rep = BitSetalloc(0, &u[0], 2, 0, 3);
-  trim(r.rep);
-}
 
 
 int BitSet::empty() const
@@ -806,6 +796,22 @@ const char* BitSettoa(const BitSet& x, char f, char t, char star)
   return fmtbase;
 }
 
+#if defined(__GNUG__) && !defined(NO_NRV)
+
+BitSet shorttoBitSet(unsigned short w) return r
+{
+  r.rep = BitSetalloc(0, &w, 1, 0, 2);  trim(r.rep);
+}
+
+BitSet longtoBitSet(unsigned long w) return r;
+{
+  unsigned short u[2];
+  u[0] = w & ((unsigned short)(~(0)));
+  u[1] = w >> BITSETBITS;
+  r.rep = BitSetalloc(0, &u[0], 2, 0, 3);
+  trim(r.rep);
+}
+
 BitSet atoBitSet(const char* s, char f, char t, char star) return r
 {
   int sl = strlen(s);
@@ -858,6 +864,81 @@ BitSet atoBitSet(const char* s, char f, char t, char star) return r
   return;
 }
 
+#else
+
+BitSet shorttoBitSet(unsigned short w) 
+{
+  BitSet r;
+  r.rep = BitSetalloc(0, &w, 1, 0, 2);  trim(r.rep);
+  return r;
+}
+
+BitSet longtoBitSet(unsigned long w)
+{
+  BitSet r;
+  unsigned short u[2];
+  u[0] = w & ((unsigned short)(~(0)));
+  u[1] = w >> BITSETBITS;
+  r.rep = BitSetalloc(0, &u[0], 2, 0, 3);
+  trim(r.rep);
+  return r;
+}
+
+BitSet atoBitSet(const char* s, char f, char t, char star) 
+{
+  BitSet r;
+  int sl = strlen(s);
+  if (sl != 0)
+  {
+    r.rep = BitSetresize(r.rep, sl / BITSETBITS + 1);
+    unsigned short* rs = r.rep->s;
+    unsigned short a = 0;
+    unsigned short m = 1;
+    char lastch = 0;
+    unsigned int i = 0;
+    unsigned int l = 1;
+    for(;;)
+    {
+      char ch = s[i];
+      if (ch == t)
+        a |= m;
+      else if (ch == star)
+      {
+        if (r.rep->virt = lastch == t)
+          *rs = a | ~(m - 1);
+        else
+          *rs = a;
+        break;
+      }
+      else if (ch != f)
+      {
+        *rs = a;
+        break;
+      }
+      lastch = ch;
+      if (++i == sl)
+      {
+        *rs = a;
+        break;
+      }
+      else if (i % BITSETBITS == 0)
+      {
+        *rs++ = a;
+        a = 0;
+        m = 1;
+        ++l;
+      }
+      else
+        m <<= 1;
+    }
+    r.rep->len = l;
+    trim(r.rep);
+  }
+  return r;
+}
+
+#endif
+
 ostream& operator << (ostream& s, const BitSet& x)
 {
   return s << BitSettoa(x);
