@@ -19,13 +19,17 @@ can know your rights and responsibilities.  It should be in a
 file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  
 */
+#ifdef __GNUG__
+#pragma implementation
+#endif
 #include <builtin.h>
 #include <Random.h>
 
-#include <DiscreteUniform.h>
+#include <DiscUnif.h>
 
 double DiscreteUniform::operator()()
 {
     long tmp = long(floor(delta * pGenerator -> asDouble()));
     return( double(pLow + tmp) );
 }
+
