@@ -20,6 +20,9 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  
 */
 
+#ifdef __GNUG__
+#pragma implementation
+#endif
 #include <Rational.h>
 #include <std.h>
 #include <math.h>
@@ -118,23 +121,6 @@ void      div(const Rational& x, const Rational& y, Rational& r)
 }
 
 
-Rational operator - (const Rational& x) return r(x)
-{
-  r.negate();
-}
-
-Rational abs(const Rational& x) return r(x)
-{
-  if (sign(r.num) < 0) r.negate();
-}
-
-
-Rational sqr(const Rational& x) return r
-{
-  mul(x.num, x.num, r.num);
-  mul(x.den, x.den, r.den);
-  r.normalize();
-}
 
 
 void Rational::invert()
@@ -198,6 +184,39 @@ Rational::Rational(double x)
   normalize();
 }
 
+
+Integer trunc(const Rational& x)
+{
+  return x.num / x.den ;
+}
+
+
+Rational pow(const Rational& x, const Integer& y)
+{
+  long yy = long(y);
+  return pow(x, yy);
+}               
+
+#if defined(__GNUG__) && !defined(NO_NRV)
+
+Rational operator - (const Rational& x) return r(x)
+{
+  r.negate();
+}
+
+Rational abs(const Rational& x) return r(x)
+{
+  if (sign(r.num) < 0) r.negate();
+}
+
+
+Rational sqr(const Rational& x) return r
+{
+  mul(x.num, x.num, r.num);
+  mul(x.den, x.den, r.den);
+  r.normalize();
+}
+
 Integer floor(const Rational& x) return q
 {
   Integer r;
@@ -226,18 +245,6 @@ Integer round(const Rational& x) return q
   }
 }
 
-Integer trunc(const Rational& x)
-{
-  return x.num / x.den ;
-}
-
-
-Rational pow(const Rational& x, const Integer& y)
-{
-  long yy = long(y);
-  return pow(x, yy);
-}               
-
 // power: no need to normalize since num & den already relatively prime
 
 Rational pow(const Rational& x, long y) return r
@@ -260,6 +267,87 @@ Rational pow(const Rational& x, long y) return r
   }
 }
 
+#else
+
+Rational operator - (const Rational& x) 
+{
+  Rational r(x); r.negate(); return r;
+}
+
+Rational abs(const Rational& x) 
+{
+  Rational r(x);
+  if (sign(r.num) < 0) r.negate();
+  return r;
+}
+
+
+Rational sqr(const Rational& x)
+{
+  Rational r;
+  mul(x.num, x.num, r.num);
+  mul(x.den, x.den, r.den);
+  r.normalize();
+  return r;
+}
+
+Integer floor(const Rational& x)
+{
+  Integer q;
+  Integer r;
+  divide(x.num, x.den, q, r);
+  if (sign(x.num) < 0 && sign(r) != 0) q--;
+  return q;
+}
+
+Integer ceil(const Rational& x)
+{
+  Integer q;
+  Integer  r;
+  divide(x.num, x.den, q, r);
+  if (sign(x.num) >= 0 && sign(r) != 0) q++;
+  return q;
+}
+
+Integer round(const Rational& x) 
+{
+  Integer q;
+  Integer r;
+  divide(x.num, x.den, q, r);
+  r <<= 1;
+  if (ucompare(r, x.den) >= 0)
+  {
+    if (sign(x.num) >= 0)
+      q++;
+    else
+      q--;
+  }
+  return q;
+}
+
+Rational pow(const Rational& x, long y)
+{
+  Rational r;
+  if (y >= 0)
+  {
+    pow(x.num, y, r.num);
+    pow(x.den, y, r.den);
+  }
+  else
+  {
+    y = -y;
+    pow(x.num, y, r.den);
+    pow(x.den, y, r.num);
+    if (sign(r.den) < 0)
+    {
+      r.num.negate();
+      r.den.negate();
+    }
+  }
+  return r;
+}
+
+#endif
 
 ostream& operator << (ostream& s, const Rational& y)
 {
@@ -303,4 +391,3 @@ int Rational::OK() const
   if (!v) error("invariant failure");
   return v;
 }
-
