@@ -20,6 +20,9 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  
 */
 
+#ifdef __GNUG__
+#pragma implementation
+#endif
 #include <builtin.h>
 #include <AllocRing.h>
 
