@@ -35,6 +35,7 @@ and this notice must be preserved on all copies.
 
 /* History
 
+
    Tue Jan 16 04:54:27 1990  Doug Lea  (dl at g.oswego.edu)
 
      version 1 released in libg++
@@ -54,6 +55,39 @@ and this notice must be preserved on all copies.
 
      hardwired SBRK_UNIT to 4096 for uniformity across systems
 
+   Tue Mar 20 14:18:23 1990  Doug Lea  (dl at g.oswego.edu)
+
+     calloc and cfree now correctly parameterized.
+
+   Sun Apr  1 10:00:48 1990  Doug Lea  (dl at g.oswego.edu)
+
+     added memalign and valloc.
+
+   Sun Jun 24 05:46:48 1990  Doug Lea  (dl at g.oswego.edu)
+
+     #include gepagesize.h only ifndef sun
+     cache pagesize after first call
+
+   Wed Jul 25 08:35:19 1990  Doug Lea  (dl at g.oswego.edu)
+
+     No longer rely on a `designated victim':
+
+       1. It sometimes caused splits of large chunks
+          when smaller ones would do, leading to
+          bad worst-case fragmentation.
+
+       2. Scanning through the av array fast anyway,
+          so the overhead isn't worth it.
+
+     To compensate, several other minor changes:
+
+       1. Unusable chunks are checked for consolidation during
+          searches inside bins, better distributing chunks
+          across bins.
+
+       2. Chunks are returned when found in malloc_find_space,
+           rather than finishing cleaning everything up, to
+           avoid wasted iterations due to (1).
 */
 
 /* 
@@ -116,17 +150,19 @@ and this notice must be preserved on all copies.
 
   On allocation, the bin corresponding to the request size is
   scanned, and if there is a chunk with size >= requested, it
-  is split, if too big, and used.
+  is split, if too big, and used. Chunks on the list which are
+  too small are examined for consolidation during this traversal.
 
-  If no chunk exists in the list, a bigger chunk is found. To minimize 
-  unsuccessful scanning of a large number of probably empty bins
-  (as happens when so many bins are used), a single designated
-  ``victim'' ibin s maintained, and is considered first. The designated
-  victim is that holding the last remaindered or consolidated chunk.
-  If the victim is unusable (too small), then a new victim is
-  searched for by scanning bigger bins. 
+  If no chunk exists in the list bigger bins are scanned in search of
+  a victim.
 
-  If the victim is usable, it is split. The remainder is placed on
+  If no victim can be found, then smaller bins are examined for
+  consolidation in order to construct a victim.
+
+  Finally, if consolidation fails to come up with a usable chunk,
+  more space is obtained from the system.
+
+  After a split, the remainder is placed on
   the back of the appropriate bin list. (All freed chunks are placed
   on fronts of lists. All remaindered or consolidated chunks are
   placed on the rear. Correspondingly, searching within a bin
@@ -135,21 +171,23 @@ and this notice must be preserved on all copies.
   bin: returned chunks vs unallocated chunks, but without the overhead 
   of maintaining 2 lists.)
 
-  If no victim can be found, then smaller bins are examined for
-  consolidation.
-
-  Finally, if consolidation fails to come up with a usable chunk,
-  more space is obtained from the system.
-
   Deallocation (free) consists only of placing the chunk on
   a list.
 
   Reallocation proceeds in the usual way. If a chunk can be extended,
   it is, else a malloc-copy-free sequence is taken.
 
+  memalign requests more than enough space from malloc, finds a
+  spot within that chunk that meets the alignment request, and
+  then possibly frees the leading and trailing space. Overreliance
+  on memalign is a sure way to fragment space.
+
+
   Some other implementation matters:
 
-  8 byte alignment is currently hardwired into the design.
+  8 byte alignment is currently hardwired into the design. Calling
+  memalign will return a chunk that is both 8-byte aligned, and
+  meets the requested alignment.
 
   The basic overhead of a used chunk is 8 bytes: 4 at the front and
   4 at the end.
@@ -209,18 +247,14 @@ extern "C" {
 extern void*     memset(void*, int, int);
 extern void*     memcpy(void*,  const void*, int);
 inline void      bzero(void* s, int l) { memset(s, 0, l); }
-inline void      bcopy(const void* s, void* d, int l) { memcpy(d, s, l); }
 #else
-extern void      bcopy(void*, void*, unsigned int);
 extern void      bzero(void*, unsigned int);
 #endif
 
+extern void      bcopy(void*, void*, unsigned int);
+
 extern void*     sbrk(unsigned int);
 
-#ifndef __cplusplus
-extern int       fputs(const char*, FILE*);
-extern int       fprintf(FILE*, const char*, ...);
-#endif
 
 #ifdef __GNUC__
 extern volatile void abort();
@@ -510,6 +544,39 @@ static void do_free_stats(const mchunkptr p)
 
 
 
+/* Utilities needed below for memalign */
+/* This is redundant with libg++ support, but not if used stand-alone */
+
+static unsigned int gcd(unsigned int a, unsigned int b)
+{
+  unsigned int tmp;
+  
+  if (b > a)
+  {
+    tmp = a; a = b; b = tmp;
+  }
+  for(;;)
+  {
+    if (b == 0)
+      return a;
+    else if (b == 1)
+      return b;
+    else
+    {
+      tmp = b;
+      b = a % b;
+      a = tmp;
+    }
+  }
+}
+
+static inline unsigned int lcm(unsigned int x, unsigned int y)
+{
+  return x / gcd(x, y) * y;
+}
+
+
+
 /* maintaining INUSE via size field */
 
 
@@ -571,13 +638,6 @@ static inline mchunkptr mem2chunk(void* mem)
 static mbinptr malloc_maxbin = FIRSTBIN;
 
 
-/* The designated victim bin. */
-
-static mbinptr malloc_victim = FIRSTBIN;
-
-
-
-
 /* operations on lists inside bins */
 
 
@@ -616,9 +676,6 @@ static inline void split(mchunkptr p, unsigned int offset)
     /* adjust maxbin (h == b means was empty) */
     if (h == b && bn > malloc_maxbin) malloc_maxbin = bn; 
 
-    /* use as next victim */
-    malloc_victim = bn;
-    
     /* adjust size of chunk to be returned */
     p->size = *((int*)((char*)(p) + offset  - SIZE_SZ)) = offset;
 
@@ -641,8 +698,6 @@ static inline void consollink(mchunkptr p)
 
   if (h == b && bn > malloc_maxbin) malloc_maxbin = bn; 
 
-  malloc_victim = bn;        /* use as next victim */
-
   UPDATE_STATS(++n_avail);
 }
 
@@ -757,7 +812,11 @@ static mchunkptr malloc_from_sys(unsigned nb)
   *last_sbrk_end = SIZE_SZ | INUSE;
 
   UPDATE_STATS((++n_avail, ++n_malloc_chunks));
-  
+
+  /* make it safe to unlink in malloc */
+  UPDATE_STATS(++n_avail);
+  p->fd = p->bk = p;
+
   return p;
 }
 
@@ -774,7 +833,6 @@ static mchunkptr malloc_from_sys(unsigned nb)
 static mchunkptr malloc_find_space(unsigned int nb)
 {
   mbinptr b;
-  mchunkptr usable = 0;
 
   /* first, re-adjust max used bin */
 
@@ -792,18 +850,21 @@ static mchunkptr malloc_find_space(unsigned int nb)
     if (b->dirty)
     {
       mchunkptr h = &(b->hd);         /* head of list */
-      mchunkptr p = h->bk;            /* chunk traverser */
+      mchunkptr p = h->fd;            /* chunk traverser */
 
       while (p != h)
       {
-        mchunkptr nextp = p->bk;       /* save, in case of relinks */
+        mchunkptr nextp = p->fd;       /* save, in case of relinks */
         int consolidated = 0;          /* only unlink/relink if consolidated */
 
         mchunkptr t;
+
+        UPDATE_STATS(++n_malloc_chunks);
+
         while (!inuse(t = prev_chunk(p))) /* consolidate backward */
         {
           if (!consolidated) { consolidated = 1; unlink(p); }
-          if (t == nextp) nextp = t->bk;
+          if (t == nextp) nextp = t->fd;
           unlink(t);
           set_size(t, t->size + p->size);
           p = t;
@@ -813,32 +874,31 @@ static mchunkptr malloc_find_space(unsigned int nb)
         while (!inuse(t = next_chunk(p))) /* consolidate forward */
         {
           if (!consolidated) { consolidated = 1; unlink(p); }
-          if (t == nextp) nextp = t->bk;
+          if (t == nextp) nextp = t->fd;
           unlink(t);
           set_size(p, p->size + t->size);
           UPDATE_STATS (++n_consol);
         }
 
-        if (consolidated)
-        {
-          consollink(p);
+       if (consolidated)
+       {
           if (p->size >= nb)
-            usable = p;
+          {
+            /* make it safe to unlink in malloc */
+            UPDATE_STATS(++n_avail);
+            p->fd = p->bk = p;
+            return p;
+          }
+          else
+            consollink(p);
         }
 
-        UPDATE_STATS(++n_malloc_chunks);
         p = nextp;
 
       }
 
       b->dirty = 0;
 
-      if (usable != 0) 
-      {
-        unlink(usable);
-        return usable;
-      }
-
     }
   }
 
@@ -856,81 +916,82 @@ void* malloc(unsigned int bytes)
   unsigned int nb  = request2size(bytes);  /* padded request size */
   mbinptr      b   = size2bin(nb);         /* corresponding bin */
   mchunkptr    hd  = &(b->hd);             /* head of its list */
-  mchunkptr    p;                          /* chunk traverser */
+  mchunkptr    p   = hd->fd;               /* chunk traverser */
 
   UPDATE_STATS((requested_mem+=bytes, ++n_malloc_bins));
 
   /* Try a (near) exact match in own bin */
+  /* clean out unusable but consolidatable chunks in bin while traversing */
 
-  for (p = hd->fd; p != hd; p = p->fd)
+  while (p != hd)
   {
     UPDATE_STATS(++n_malloc_chunks);
-
     if (p->size >= nb)
-    {
-      unlink(p);
       goto found;
-    }
-  }
-
-  /* No exact match, so select a victim */
-
-  UPDATE_STATS(++n_malloc_chunks);
-
-  p = malloc_victim->hd.bk; 
-  if (p->size >= nb)
-    unlink(p);
-  else
-  {
-    /* Find a new victim. */
-
-    for (;;)
+    else    /* try to consolidate; same code as malloc_find_space */
     {
-      UPDATE_STATS(++n_malloc_bins);
-      ++b;
-      p = b->hd.bk;
-      if (p != &(b->hd)) 
+      mchunkptr nextp = p->fd;       /* save, in case of relinks */
+      int consolidated = 0;          /* only unlink/relink if consolidated */
+      
+      mchunkptr t;
+
+      while (!inuse(t = prev_chunk(p))) /* consolidate backward */
       {
-        /* p must be usable since it's in bigger bin */
-        /* Consolidate, so it can serve as a victim longer */
-
-        mchunkptr t;
-
-        unlink(p);
-
-        while (!inuse(t = prev_chunk(p))) /* consolidate backward */
-        {
-          unlink(t);
-          set_size(t, t->size + p->size);
-          p = t;
-          UPDATE_STATS (++n_consol);
-        }
-        
-        while (!inuse(t = next_chunk(p))) /* consolidate forward */
-        {
-          unlink(t);
-          set_size(p, p->size + t->size);
-          UPDATE_STATS (++n_consol);
-        }
-
-        malloc_victim = b;  /* record this bin for next time */
-        break;
+        if (!consolidated) { consolidated = 1; unlink(p); }
+        if (t == nextp) nextp = t->fd;
+        unlink(t);
+        set_size(t, t->size + p->size);
+        p = t;
+        UPDATE_STATS (++n_consol);
       }
-      else if (b > malloc_maxbin)
+      
+      while (!inuse(t = next_chunk(p))) /* consolidate forward */
       {
-        p = malloc_find_space(nb);
-        if (p == 0)
-          return 0;
+        if (!consolidated) { consolidated = 1; unlink(p); }
+        if (t == nextp) nextp = t->fd;
+        unlink(t);
+        set_size(p, p->size + t->size);
+        UPDATE_STATS (++n_consol);
+      }
+      
+      if (consolidated)
+      {
+        if (p->size >= nb)
+        {
+          /* make it safe to unlink again below */
+          UPDATE_STATS(++n_avail);
+          p->fd = p->bk = p;
+          goto found;
+        }
         else
-          break;
+          consollink(p);
       }
+
+      p = nextp;
+
     }
   }
 
-  /* use what we found */
+  b->dirty = 0; /* true if got here */
 
- found:
+  /*  Scan bigger bins for a victim */
 
+  while (++b <= malloc_maxbin)
+  {
+    UPDATE_STATS(++n_malloc_bins);
+    if ((p = b->hd.bk) != &(b->hd))    /* no need to check size */
+      goto found;
+  }
+
+  /* Consolidate or sbrk */
+
+  p = malloc_find_space(nb);
+
+  if (p == 0) return 0; /* allocation failure */
+
+ found:   /* Use what we found */
+
+  unlink(p);
   split(p, nb); 
   UPDATE_STATS(do_malloc_stats(p));
   return chunk2mem(p);
@@ -950,7 +1011,7 @@ void free(void* mem)
 }
 
 
-void* calloc(unsigned int elem_size, unsigned int n)
+void* calloc(unsigned int n, unsigned int elem_size)
 {
   unsigned int sz = n * elem_size;
   void* p = malloc(sz);
@@ -959,7 +1020,7 @@ void* calloc(unsigned int elem_size, unsigned int n)
 };
 
 /* This is here for compatibility with older systems */
-void cfree(void *mem, unsigned int n, unsigned int elem_size)
+void cfree(void *mem)
 {
   free(mem);
 }
@@ -1008,12 +1069,8 @@ void* realloc(void* mem, unsigned int bytes)
     room = p->size - nb;
     if (room >= 0)
     {
-      if (room >= MINSIZE)
-      {
-        split(p, nb);
-        UPDATE_STATS(malloced_mem -= room);
-      }
-      
+      split(p, nb);
+      UPDATE_STATS(malloced_mem -= room);
       return chunk2mem(p);
     }
     else /* do the obvious */
@@ -1031,6 +1088,91 @@ void* realloc(void* mem, unsigned int bytes)
 
 
 
+/* return a pointer to space with at least the alignment requested */
+
+void* memalign(unsigned int alignment, unsigned int bytes)
+{
+  mchunkptr p;
+  unsigned int nb = request2size(bytes);
+
+  /* find an alignment that both we and the user can live with: */
+  /* least common multiple guarantees mutual happiness */
+  unsigned int align = lcm(alignment, MALLOC_MIN_OVERHEAD);
+  unsigned int mask = align - 1;
+
+  /* call malloc with worst case padding to hit alignment; */
+  /* we will give back extra */
+
+  unsigned int req = nb + align + MINSIZE;
+  void* m = malloc(req);
+
+  if (m == 0) return m;
+
+  p = mem2chunk(m);
+
+  /* keep statistics on track */
+
+  UPDATE_STATS(--n_mallocs);
+  UPDATE_STATS(malloced_mem -= p->size);
+  UPDATE_STATS(requested_mem -= req);
+  UPDATE_STATS(requested_mem += bytes);
+
+  if (((int)(m) & (mask)) != 0) /* misaligned */
+  {
+
+    /* find an aligned spot inside chunk */
+
+    mchunkptr ap = (mchunkptr)(( ((int)(m) + mask) & -align) - SIZE_SZ);
+
+    unsigned int gap = (unsigned int)(ap) - (unsigned int)(p);
+    unsigned int room;
+
+    /* we need to give back leading space in a chunk of at least MINSIZE */
+
+    if (gap < MINSIZE)
+    {
+      /* This works since align >= MINSIZE */
+      /* and we've malloc'd enough total room */
+
+      ap = (mchunkptr)( (int)(ap) + align );
+      gap += align;    
+    }
+
+    if (gap + nb > p->size) /* can't happen unless chunk sizes corrupted */
+      malloc_user_error();
+
+    room = p->size - gap;
+
+    /* give back leader */
+    set_size(p, gap);
+    consollink(p);
+
+    /* use the rest */
+    p = ap;
+    set_size(p, room);
+  }
+
+  /* also give back spare room at the end */
+
+  split(p, nb); 
+  UPDATE_STATS(do_malloc_stats(p));
+  return chunk2mem(p);
+
+}
+
+#ifndef sun
+#include <getpagesize.h>
+#endif
+
+static unsigned int malloc_pagesize = 0;
+
+void* valloc(unsigned int bytes)
+{
+  if (malloc_pagesize == 0) malloc_pagesize = getpagesize();
+  return memalign (malloc_pagesize, bytes);
+}
+    
+
 void malloc_stats()
 {
 #ifndef MALLOC_STATS
