@@ -24,6 +24,9 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  
 */
 
+#ifdef __GNUG__
+#pragma implementation
+#endif
 #include <stddef.h>
 #include <malloc.h>
 
