@@ -113,6 +113,9 @@ what you give them.   Help stamp out software-hoarding!  */
 /* The `emacs' switch turns on certain special matching commands
  that make sense only in emacs. */
 
+#ifdef __GNUG__
+#pragma implementation
+#endif
 #include "config.h"
 #include "lisp.h"
 #include "buffer.h"
@@ -169,17 +172,6 @@ init_syntax_once (void)
 
 #include "regex.h"
 
-/* Make alloca work the best possible way.  */
-#ifndef alloca
-#ifdef __GNUC__
-#define alloca __builtin_alloca
-#else
-#ifdef sparc
-#include <alloca.h>
-#endif
-#endif
-#endif
-
 /* Number of failure points to allocate space for initially,
  when matching.  If this number is exceeded, more space is allocated,
  so it is not a hard limit.  */
